﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;

namespace WpfApplication8
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            EventManager.RegisterClassHandler(typeof(Infragistics.Windows.Editors.XamComboEditor), Infragistics.Windows.Editors.XamComboEditor.ValueChangedEvent,
                new RoutedPropertyChangedEventHandler<object>(OnValueEditorValueChanged));
        }

        void OnValueEditorValueChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            (sender as Infragistics.Windows.Editors.ValueEditor).EndEditMode(true, true);
        }
    }

    public class DataViewModel : DependencyObject
    {
        public ObservableCollection<DataClass> Data
        {
            get { return (ObservableCollection<DataClass>)GetValue(DataProperty); }
            set { SetValue(DataProperty, value); }
        }
        public static readonly DependencyProperty DataProperty =
            DependencyProperty.Register("Data", typeof(ObservableCollection<DataClass>), typeof(DataViewModel), new UIPropertyMetadata(new ObservableCollection<DataClass>()));
        public Array Operations
        {
            get { return Enum.GetValues(typeof(Operations)); }
        }
        public DataViewModel()
        {
            this.Data = new ObservableCollection<DataClass>();
            this.Data.Add(new DataClass() { DataId = 0, Description = "item 0" });
            this.Data.Add(new DataClass() { DataId = 1, Description = "item 1" });
            this.Data.Add(new DataClass() { DataId = 2, Description = "item 2" });
            this.Data.Add(new DataClass() { DataId = 3, Description = "item 3" });
        }
    }

    public class DataClass : DependencyObject
    {
        public int DataId
        {
            get { return (int)GetValue(DataIdProperty); }
            set { SetValue(DataIdProperty, value); }
        }
        public static readonly DependencyProperty DataIdProperty =
            DependencyProperty.Register("DataId", typeof(int), typeof(DataClass), new UIPropertyMetadata(0));

        public string Description
        {
            get { return (string)GetValue(DescriptionProperty); }
            set { SetValue(DescriptionProperty, value); }
        }
        public static readonly DependencyProperty DescriptionProperty =
            DependencyProperty.Register("Description", typeof(string), typeof(DataClass), new UIPropertyMetadata(string.Empty));

        public Operations? PerformOperation
        {
            get { return (Operations?)GetValue(PerformOperationProperty); }
            set { SetValue(PerformOperationProperty, value); }
        }
        public static readonly DependencyProperty PerformOperationProperty =
            DependencyProperty.Register("PerformOperation", typeof(Operations?), typeof(DataClass), new FrameworkPropertyMetadata(null,new PropertyChangedCallback(OnPerformOperationChanged)));
        private static void OnPerformOperationChanged(DependencyObject d, DependencyPropertyChangedEventArgs args)
        {
            DataClass dataClass = d as DataClass;
            Operations? operation = (Operations?)args.NewValue;


            if (operation != Operations.DoNotSave || operation != Operations.Default)
            {
                dataClass.IsPerformingOperation = true;
            }
            else
            {
                dataClass.IsPerformingOperation = false;
            }
        }

        public bool IsPerformingOperation
        {
            get { return (bool)GetValue(IsPerformingOperationProperty); }
            set { SetValue(IsPerformingOperationProperty, value); }
        }
        public static readonly DependencyProperty IsPerformingOperationProperty =
            DependencyProperty.Register("IsPerformingOperation", typeof(bool), typeof(DataClass), new FrameworkPropertyMetadata(false,new PropertyChangedCallback(OnIsPerformingOperationChanged)));
        private static void OnIsPerformingOperationChanged(DependencyObject d, DependencyPropertyChangedEventArgs args)
        {
            DataClass dataClass = d as DataClass;
            bool isPerformingOperation = (bool)args.NewValue;

            if (isPerformingOperation)
            {
                dataClass.SetValue(DescriptionProperty, "Executing Operation " + dataClass.PerformOperation);
                PerformLongOperation(dataClass);
            }
            else
            {
                dataClass.SetValue(DescriptionProperty, "Operation " + dataClass.PerformOperation + " Completed!");
            }
        }

        private static void LongOperation(object obj)
        {
            System.Threading.Thread.Sleep(new Random().Next(5000, 10000));
            (obj as DataClass).Dispatcher.BeginInvoke(new Action(
         () =>
         {
             (obj as DataClass).IsPerformingOperation = false;
         }), null);
        }
        private static void PerformLongOperation(DataClass obj)
        {
            System.Threading.Thread thread = new System.Threading.Thread(new System.Threading.ParameterizedThreadStart(LongOperation));
            thread.Start(obj);
        }
    }

    public enum Operations
    {
        SaveLocal, SaveHere, SaveThere, DoNotSave, Default
    }
}
